@@ -6,9 +6,9 @@ import (
 	"strings"
 
 	"github.com/jesseduffield/gocui"
-	"github.com/jesseduffield/lazygit/pkg/commands"
+	"github.com/jesseduffield/lazygit/pkg/commands/git_commands"
+	"github.com/jesseduffield/lazygit/pkg/commands/loaders"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
-	"github.com/jesseduffield/lazygit/pkg/commands/oscommands"
 	"github.com/jesseduffield/lazygit/pkg/config"
 	"github.com/jesseduffield/lazygit/pkg/gui/filetree"
 	"github.com/jesseduffield/lazygit/pkg/utils"
@@ -42,9 +42,7 @@ func (gui *Gui) getSelectedPath() string {
 	return node.GetPath()
 }
 
-func (gui *Gui) selectFile(alreadySelected bool) error {
-	gui.Views.Files.FocusPoint(0, gui.State.Panels.Files.SelectedLineIdx)
-
+func (gui *Gui) filesRenderToMain() error {
 	node := gui.getSelectedFileNode()
 
 	if node == nil {
@@ -56,30 +54,24 @@ func (gui *Gui) selectFile(alreadySelected bool) error {
 		})
 	}
 
-	if !alreadySelected {
-		gui.takeOverMergeConflictScrolling()
-	}
-
 	if node.File != nil && node.File.HasInlineMergeConflicts {
 		return gui.refreshMergePanelWithLock()
 	}
 
-	cmdStr := gui.GitCommand.WorktreeFileDiffCmdStr(node, false, !node.GetHasUnstagedChanges() && node.GetHasStagedChanges(), gui.State.IgnoreWhitespaceInDiffView)
-	cmd := gui.OSCommand.ExecutableFromString(cmdStr)
+	cmdObj := gui.Git.WorkingTree.WorktreeFileDiffCmdObj(node, false, !node.GetHasUnstagedChanges() && node.GetHasStagedChanges(), gui.State.IgnoreWhitespaceInDiffView)
 
 	refreshOpts := refreshMainOpts{main: &viewUpdateOpts{
 		title: gui.Tr.UnstagedChanges,
-		task:  NewRunPtyTask(cmd),
+		task:  NewRunPtyTask(cmdObj.GetCmd()),
 	}}
 
 	if node.GetHasUnstagedChanges() {
 		if node.GetHasStagedChanges() {
-			cmdStr := gui.GitCommand.WorktreeFileDiffCmdStr(node, false, true, gui.State.IgnoreWhitespaceInDiffView)
-			cmd := gui.OSCommand.ExecutableFromString(cmdStr)
+			cmdObj := gui.Git.WorkingTree.WorktreeFileDiffCmdObj(node, false, true, gui.State.IgnoreWhitespaceInDiffView)
 
 			refreshOpts.secondary = &viewUpdateOpts{
 				title: gui.Tr.StagedChanges,
-				task:  NewRunPtyTask(cmd),
+				task:  NewRunPtyTask(cmdObj.GetCmd()),
 			}
 		}
 	} else {
@@ -121,9 +113,12 @@ func (gui *Gui) refreshFilesAndSubmodules() error {
 		if gui.currentContext().GetKey() == FILES_CONTEXT_KEY || (g.CurrentView() == gui.Views.Main && ContextKey(g.CurrentView().Context) == MAIN_MERGING_CONTEXT_KEY) {
 			newSelectedPath := gui.getSelectedPath()
 			alreadySelected := selectedPath != "" && newSelectedPath == selectedPath
-			if err := gui.selectFile(alreadySelected); err != nil {
-				return err
+			if !alreadySelected {
+				gui.takeOverMergeConflictScrolling()
 			}
+
+			gui.Views.Files.FocusPoint(0, gui.State.Panels.Files.SelectedLineIdx)
+			return gui.filesRenderToMain()
 		}
 
 		return nil
@@ -162,14 +157,14 @@ func (gui *Gui) stageSelectedFile() error {
 		return nil
 	}
 
-	return gui.GitCommand.StageFile(file.Name)
+	return gui.Git.WorkingTree.StageFile(file.Name)
 }
 
 func (gui *Gui) handleEnterFile() error {
-	return gui.enterFile(false, -1)
+	return gui.enterFile(OnFocusOpts{ClickedViewName: "", ClickedViewLineIdx: -1})
 }
 
-func (gui *Gui) enterFile(forceSecondaryFocused bool, selectedLineIdx int) error {
+func (gui *Gui) enterFile(opts OnFocusOpts) error {
 	node := gui.getSelectedFileNode()
 	if node == nil {
 		return nil
@@ -193,9 +188,8 @@ func (gui *Gui) enterFile(forceSecondaryFocused bool, selectedLineIdx int) error
 	if file.HasMergeConflicts {
 		return gui.createErrorPanel(gui.Tr.FileStagingRequirements)
 	}
-	_ = gui.pushContext(gui.State.Contexts.Staging)
 
-	return gui.handleRefreshStagingPanel(forceSecondaryFocused, selectedLineIdx) // TODO: check if this is broken, try moving into context code
+	return gui.pushContext(gui.State.Contexts.Staging, opts)
 }
 
 func (gui *Gui) handleFilePress() error {
@@ -212,11 +206,13 @@ func (gui *Gui) handleFilePress() error {
 		}
 
 		if file.HasUnstagedChanges {
-			if err := gui.GitCommand.WithSpan(gui.Tr.Spans.StageFile).StageFile(file.Name); err != nil {
+			gui.logAction(gui.Tr.Actions.StageFile)
+			if err := gui.Git.WorkingTree.StageFile(file.Name); err != nil {
 				return gui.surfaceError(err)
 			}
 		} else {
-			if err := gui.GitCommand.WithSpan(gui.Tr.Spans.UnstageFile).UnStageFile(file.Names(), file.Tracked); err != nil {
+			gui.logAction(gui.Tr.Actions.UnstageFile)
+			if err := gui.Git.WorkingTree.UnStageFile(file.Names(), file.Tracked); err != nil {
 				return gui.surfaceError(err)
 			}
 		}
@@ -228,12 +224,14 @@ func (gui *Gui) handleFilePress() error {
 		}
 
 		if node.GetHasUnstagedChanges() {
-			if err := gui.GitCommand.WithSpan(gui.Tr.Spans.StageFile).StageFile(node.Path); err != nil {
+			gui.logAction(gui.Tr.Actions.StageFile)
+			if err := gui.Git.WorkingTree.StageFile(node.Path); err != nil {
 				return gui.surfaceError(err)
 			}
 		} else {
 			// pretty sure it doesn't matter that we're always passing true here
-			if err := gui.GitCommand.WithSpan(gui.Tr.Spans.UnstageFile).UnStageFile([]string{node.Path}, true); err != nil {
+			gui.logAction(gui.Tr.Actions.UnstageFile)
+			if err := gui.Git.WorkingTree.UnStageFile([]string{node.Path}, true); err != nil {
 				return gui.surfaceError(err)
 			}
 		}
@@ -243,7 +241,7 @@ func (gui *Gui) handleFilePress() error {
 		return err
 	}
 
-	return gui.selectFile(true)
+	return gui.State.Contexts.Files.HandleFocus()
 }
 
 func (gui *Gui) allFilesStaged() bool {
@@ -255,16 +253,19 @@ func (gui *Gui) allFilesStaged() bool {
 	return true
 }
 
-func (gui *Gui) focusAndSelectFile() error {
-	return gui.selectFile(false)
+func (gui *Gui) onFocusFile() error {
+	gui.takeOverMergeConflictScrolling()
+	return nil
 }
 
 func (gui *Gui) handleStageAll() error {
 	var err error
 	if gui.allFilesStaged() {
-		err = gui.GitCommand.WithSpan(gui.Tr.Spans.UnstageAllFiles).UnstageAll()
+		gui.logAction(gui.Tr.Actions.UnstageAllFiles)
+		err = gui.Git.WorkingTree.UnstageAll()
 	} else {
-		err = gui.GitCommand.WithSpan(gui.Tr.Spans.StageAllFiles).StageAll()
+		gui.logAction(gui.Tr.Actions.StageAllFiles)
+		err = gui.Git.WorkingTree.StageAll()
 	}
 	if err != nil {
 		_ = gui.surfaceError(err)
@@ -274,7 +275,7 @@ func (gui *Gui) handleStageAll() error {
 		return err
 	}
 
-	return gui.selectFile(false)
+	return gui.State.Contexts.Files.HandleFocus()
 }
 
 func (gui *Gui) handleIgnoreFile() error {
@@ -287,12 +288,10 @@ func (gui *Gui) handleIgnoreFile() error {
 		return gui.createErrorPanel("Cannot ignore .gitignore")
 	}
 
-	gitCommand := gui.GitCommand.WithSpan(gui.Tr.Spans.IgnoreFile)
-
 	unstageFiles := func() error {
 		return node.ForEachFile(func(file *models.File) error {
 			if file.HasStagedChanges {
-				if err := gitCommand.UnStageFile(file.Names(), file.Tracked); err != nil {
+				if err := gui.Git.WorkingTree.UnStageFile(file.Names(), file.Tracked); err != nil {
 					return err
 				}
 			}
@@ -306,16 +305,17 @@ func (gui *Gui) handleIgnoreFile() error {
 			title:  gui.Tr.IgnoreTracked,
 			prompt: gui.Tr.IgnoreTrackedPrompt,
 			handleConfirm: func() error {
+				gui.logAction(gui.Tr.Actions.IgnoreFile)
 				// not 100% sure if this is necessary but I'll assume it is
 				if err := unstageFiles(); err != nil {
 					return err
 				}
 
-				if err := gitCommand.RemoveTrackedFiles(node.GetPath()); err != nil {
+				if err := gui.Git.WorkingTree.RemoveTrackedFiles(node.GetPath()); err != nil {
 					return err
 				}
 
-				if err := gitCommand.Ignore(node.GetPath()); err != nil {
+				if err := gui.Git.WorkingTree.Ignore(node.GetPath()); err != nil {
 					return err
 				}
 				return gui.refreshSidePanels(refreshOptions{scope: []RefreshableView{FILES}})
@@ -323,11 +323,13 @@ func (gui *Gui) handleIgnoreFile() error {
 		})
 	}
 
+	gui.logAction(gui.Tr.Actions.IgnoreFile)
+
 	if err := unstageFiles(); err != nil {
 		return err
 	}
 
-	if err := gitCommand.Ignore(node.GetPath()); err != nil {
+	if err := gui.Git.WorkingTree.Ignore(node.GetPath()); err != nil {
 		return gui.surfaceError(err)
 	}
 
@@ -335,7 +337,7 @@ func (gui *Gui) handleIgnoreFile() error {
 }
 
 func (gui *Gui) handleWIPCommitPress() error {
-	skipHookPrefix := gui.Config.GetUserConfig().Git.SkipHookPrefix
+	skipHookPrefix := gui.UserConfig.Git.SkipHookPrefix
 	if skipHookPrefix == "" {
 		return gui.createErrorPanel(gui.Tr.SkipHookPrefixNotConfigured)
 	}
@@ -349,7 +351,7 @@ func (gui *Gui) handleWIPCommitPress() error {
 }
 
 func (gui *Gui) commitPrefixConfigForRepo() *config.CommitPrefixConfig {
-	cfg, ok := gui.Config.GetUserConfig().Git.CommitPrefixes[utils.GetCurrentRepoName()]
+	cfg, ok := gui.UserConfig.Git.CommitPrefixes[utils.GetCurrentRepoName()]
 	if !ok {
 		return nil
 	}
@@ -359,8 +361,9 @@ func (gui *Gui) commitPrefixConfigForRepo() *config.CommitPrefixConfig {
 
 func (gui *Gui) prepareFilesForCommit() error {
 	noStagedFiles := len(gui.stagedFiles()) == 0
-	if noStagedFiles && gui.Config.GetUserConfig().Gui.SkipNoStagedFilesWarning {
-		err := gui.GitCommand.WithSpan(gui.Tr.Spans.StageAllFiles).StageAll()
+	if noStagedFiles && gui.UserConfig.Gui.SkipNoStagedFilesWarning {
+		gui.logAction(gui.Tr.Actions.StageAllFiles)
+		err := gui.Git.WorkingTree.StageAll()
 		if err != nil {
 			return err
 		}
@@ -384,18 +387,24 @@ func (gui *Gui) handleCommitPress() error {
 		return gui.promptToStageAllAndRetry(gui.handleCommitPress)
 	}
 
-	commitPrefixConfig := gui.commitPrefixConfigForRepo()
-	if commitPrefixConfig != nil {
-		prefixPattern := commitPrefixConfig.Pattern
-		prefixReplace := commitPrefixConfig.Replace
-		rgx, err := regexp.Compile(prefixPattern)
-		if err != nil {
-			return gui.createErrorPanel(fmt.Sprintf("%s: %s", gui.Tr.LcCommitPrefixPatternError, err.Error()))
-		}
-		prefix := rgx.ReplaceAllString(gui.getCheckedOutBranch().Name, prefixReplace)
+	if len(gui.State.failedCommitMessage) > 0 {
 		gui.Views.CommitMessage.ClearTextArea()
-		gui.Views.CommitMessage.TextArea.TypeString(prefix)
-		gui.render()
+		gui.Views.CommitMessage.TextArea.TypeString(gui.State.failedCommitMessage)
+		gui.Views.CommitMessage.RenderTextArea()
+	} else {
+		commitPrefixConfig := gui.commitPrefixConfigForRepo()
+		if commitPrefixConfig != nil {
+			prefixPattern := commitPrefixConfig.Pattern
+			prefixReplace := commitPrefixConfig.Replace
+			rgx, err := regexp.Compile(prefixPattern)
+			if err != nil {
+				return gui.createErrorPanel(fmt.Sprintf("%s: %s", gui.Tr.LcCommitPrefixPatternError, err.Error()))
+			}
+			prefix := rgx.ReplaceAllString(gui.getCheckedOutBranch().Name, prefixReplace)
+			gui.Views.CommitMessage.ClearTextArea()
+			gui.Views.CommitMessage.TextArea.TypeString(prefix)
+			gui.Views.CommitMessage.RenderTextArea()
+		}
 	}
 
 	gui.g.Update(func(g *gocui.Gui) error {
@@ -414,7 +423,8 @@ func (gui *Gui) promptToStageAllAndRetry(retry func() error) error {
 		title:  gui.Tr.NoFilesStagedTitle,
 		prompt: gui.Tr.NoFilesStagedPrompt,
 		handleConfirm: func() error {
-			if err := gui.GitCommand.WithSpan(gui.Tr.Spans.StageAllFiles).StageAll(); err != nil {
+			gui.logAction(gui.Tr.Actions.StageAllFiles)
+			if err := gui.Git.WorkingTree.StageAll(); err != nil {
 				return gui.surfaceError(err)
 			}
 			if err := gui.refreshFilesAndSubmodules(); err != nil {
@@ -443,9 +453,9 @@ func (gui *Gui) handleAmendCommitPress() error {
 		title:  strings.Title(gui.Tr.AmendLastCommit),
 		prompt: gui.Tr.SureToAmend,
 		handleConfirm: func() error {
-			cmdStr := gui.GitCommand.AmendHeadCmdStr()
-			gui.OnRunCommand(oscommands.NewCmdLogEntry(cmdStr, gui.Tr.Spans.AmendCommit, true))
-			return gui.withGpgHandling(cmdStr, gui.Tr.AmendingStatus, nil)
+			cmdObj := gui.Git.Commit.AmendHeadCmdObj()
+			gui.logAction(gui.Tr.Actions.AmendCommit)
+			return gui.withGpgHandling(cmdObj, gui.Tr.AmendingStatus, nil)
 		},
 	})
 }
@@ -461,8 +471,9 @@ func (gui *Gui) handleCommitEditorPress() error {
 		return gui.promptToStageAllAndRetry(gui.handleCommitEditorPress)
 	}
 
+	gui.logAction(gui.Tr.Actions.Commit)
 	return gui.runSubprocessWithSuspenseAndRefresh(
-		gui.OSCommand.WithSpan(gui.Tr.Spans.Commit).PrepareSubProcess("git", "commit"),
+		gui.Git.Commit.CommitEditorCmdObj(),
 	)
 }
 
@@ -502,13 +513,14 @@ func (gui *Gui) editFile(filename string) error {
 }
 
 func (gui *Gui) editFileAtLine(filename string, lineNumber int) error {
-	cmdStr, err := gui.GitCommand.EditFileCmdStr(filename, lineNumber)
+	cmdStr, err := gui.Git.File.GetEditCmdStr(filename, lineNumber)
 	if err != nil {
 		return gui.surfaceError(err)
 	}
 
+	gui.logAction(gui.Tr.Actions.EditFile)
 	return gui.runSubprocessWithSuspenseAndRefresh(
-		gui.OSCommand.WithSpan(gui.Tr.Spans.EditFile).ShellCommandFromString(cmdStr),
+		gui.OSCommand.Cmd.NewShell(cmdStr),
 	)
 }
 
@@ -550,7 +562,8 @@ func (gui *Gui) refreshStateFiles() error {
 	prevNodes := gui.State.FileManager.GetAllItems()
 	prevSelectedLineIdx := gui.State.Panels.Files.SelectedLineIdx
 
-	files := gui.GitCommand.GetStatusFiles(commands.GetStatusFileOptions{})
+	files := gui.Git.Loaders.Files.
+		GetStatusFiles(loaders.GetStatusFileOptions{})
 
 	// for when you stage the old file of a rename and the new file is in a collapsed dir
 	state.FileManager.RWMutex.Lock()
@@ -636,7 +649,7 @@ func (gui *Gui) handlePullFiles() error {
 		return nil
 	}
 
-	span := gui.Tr.Spans.Pull
+	action := gui.Tr.Actions.Pull
 
 	currentBranch := gui.currentBranch()
 	if currentBranch == nil {
@@ -647,41 +660,43 @@ func (gui *Gui) handlePullFiles() error {
 	// if we have no upstream branch we need to set that first
 	if !currentBranch.IsTrackingRemote() {
 		// see if we have this branch in our config with an upstream
-		conf, err := gui.GitCommand.Repo.Config()
+		branches, err := gui.Git.Config.Branches()
 		if err != nil {
 			return gui.surfaceError(err)
 		}
-		for branchName, branch := range conf.Branches {
+		for branchName, branch := range branches {
 			if branchName == currentBranch.Name {
-				return gui.pullFiles(PullFilesOptions{RemoteName: branch.Remote, BranchName: branch.Name, span: span})
+				return gui.pullFiles(PullFilesOptions{RemoteName: branch.Remote, BranchName: branch.Name, action: action})
 			}
 		}
 
+		suggestedRemote := getSuggestedRemote(gui.State.Remotes)
+
 		return gui.prompt(promptOpts{
 			title:               gui.Tr.EnterUpstream,
-			initialContent:      "origin/" + currentBranch.Name,
+			initialContent:      suggestedRemote + "/" + currentBranch.Name,
 			findSuggestionsFunc: gui.getRemoteBranchesSuggestionsFunc("/"),
 			handleConfirm: func(upstream string) error {
-				if err := gui.GitCommand.SetUpstreamBranch(upstream); err != nil {
+				if err := gui.Git.Branch.SetCurrentBranchUpstream(upstream); err != nil {
 					errorMessage := err.Error()
 					if strings.Contains(errorMessage, "does not exist") {
 						errorMessage = fmt.Sprintf("upstream branch %s not found.\nIf you expect it to exist, you should fetch (with 'f').\nOtherwise, you should push (with 'shift+P')", upstream)
 					}
 					return gui.createErrorPanel(errorMessage)
 				}
-				return gui.pullFiles(PullFilesOptions{span: span})
+				return gui.pullFiles(PullFilesOptions{action: action})
 			},
 		})
 	}
 
-	return gui.pullFiles(PullFilesOptions{span: span})
+	return gui.pullFiles(PullFilesOptions{action: action})
 }
 
 type PullFilesOptions struct {
 	RemoteName      string
 	BranchName      string
 	FastForwardOnly bool
-	span            string
+	action          string
 }
 
 func (gui *Gui) pullFiles(opts PullFilesOptions) error {
@@ -699,14 +714,13 @@ func (gui *Gui) pullWithLock(opts PullFilesOptions) error {
 	gui.Mutexes.FetchMutex.Lock()
 	defer gui.Mutexes.FetchMutex.Unlock()
 
-	gitCommand := gui.GitCommand.WithSpan(opts.span)
+	gui.logAction(opts.action)
 
-	err := gitCommand.Pull(
-		commands.PullOptions{
-			PromptUserForCredential: gui.promptUserForCredential,
-			RemoteName:              opts.RemoteName,
-			BranchName:              opts.BranchName,
-			FastForwardOnly:         opts.FastForwardOnly,
+	err := gui.Git.Sync.Pull(
+		git_commands.PullOptions{
+			RemoteName:      opts.RemoteName,
+			BranchName:      opts.BranchName,
+			FastForwardOnly: opts.FastForwardOnly,
 		},
 	)
 	if err == nil {
@@ -727,16 +741,16 @@ func (gui *Gui) push(opts pushOpts) error {
 		return err
 	}
 	go utils.Safe(func() {
-		err := gui.GitCommand.WithSpan(gui.Tr.Spans.Push).Push(commands.PushOpts{
-			Force:                   opts.force,
-			UpstreamRemote:          opts.upstreamRemote,
-			UpstreamBranch:          opts.upstreamBranch,
-			SetUpstream:             opts.setUpstream,
-			PromptUserForCredential: gui.promptUserForCredential,
+		gui.logAction(gui.Tr.Actions.Push)
+		err := gui.Git.Sync.Push(git_commands.PushOpts{
+			Force:          opts.force,
+			UpstreamRemote: opts.upstreamRemote,
+			UpstreamBranch: opts.upstreamBranch,
+			SetUpstream:    opts.setUpstream,
 		})
 
 		if err != nil && !opts.force && strings.Contains(err.Error(), "Updates were rejected") {
-			forcePushDisabled := gui.Config.GetUserConfig().Git.DisableForcePushing
+			forcePushDisabled := gui.UserConfig.Git.DisableForcePushing
 			if forcePushDisabled {
 				_ = gui.createErrorPanel(gui.Tr.UpdatesRejectedAndForcePushDisabled)
 				return
@@ -794,12 +808,14 @@ func (gui *Gui) pushFiles() error {
 			)
 		}
 
-		if gui.GitCommand.PushToCurrent {
+		suggestedRemote := getSuggestedRemote(gui.State.Remotes)
+
+		if gui.Git.Config.GetPushToCurrent() {
 			return gui.push(pushOpts{setUpstream: true})
 		} else {
 			return gui.prompt(promptOpts{
 				title:               gui.Tr.EnterUpstream,
-				initialContent:      "origin " + currentBranch.Name,
+				initialContent:      suggestedRemote + " " + currentBranch.Name,
 				findSuggestionsFunc: gui.getRemoteBranchesSuggestionsFunc(" "),
 				handleConfirm: func(upstream string) error {
 					var upstreamBranch, upstreamRemote string
@@ -824,8 +840,22 @@ func (gui *Gui) pushFiles() error {
 	}
 }
 
+func getSuggestedRemote(remotes []*models.Remote) string {
+	if len(remotes) == 0 {
+		return "origin"
+	}
+
+	for _, remote := range remotes {
+		if remote.Name == "origin" {
+			return remote.Name
+		}
+	}
+
+	return remotes[0].Name
+}
+
 func (gui *Gui) requestToForcePush() error {
-	forcePushDisabled := gui.Config.GetUserConfig().Git.DisableForcePushing
+	forcePushDisabled := gui.UserConfig.Git.DisableForcePushing
 	if forcePushDisabled {
 		return gui.createErrorPanel(gui.Tr.ForcePushDisabled)
 	}
@@ -840,12 +870,12 @@ func (gui *Gui) requestToForcePush() error {
 }
 
 func (gui *Gui) upstreamForBranchInConfig(branchName string) (string, string, error) {
-	conf, err := gui.GitCommand.Repo.Config()
+	branches, err := gui.Git.Config.Branches()
 	if err != nil {
 		return "", "", err
 	}
 
-	for configBranchName, configBranch := range conf.Branches {
+	for configBranchName, configBranch := range branches {
 		if configBranchName == branchName {
 			return configBranch.Remote, configBranchName, nil
 		}
@@ -868,7 +898,8 @@ func (gui *Gui) handleSwitchToMerge() error {
 }
 
 func (gui *Gui) openFile(filename string) error {
-	if err := gui.OSCommand.WithSpan(gui.Tr.Spans.OpenFile).OpenFile(filename); err != nil {
+	gui.logAction(gui.Tr.Actions.OpenFile)
+	if err := gui.OSCommand.OpenFile(filename); err != nil {
 		return gui.surfaceError(err)
 	}
 	return nil
@@ -900,9 +931,9 @@ func (gui *Gui) handleCustomCommand() error {
 				gui.Log.Error(err)
 			}
 
-			gui.OnRunCommand(oscommands.NewCmdLogEntry(command, gui.Tr.Spans.CustomCommand, true))
+			gui.logAction(gui.Tr.Actions.CustomCommand)
 			return gui.runSubprocessWithSuspenseAndRefresh(
-				gui.OSCommand.PrepareShellSubProcess(command),
+				gui.OSCommand.Cmd.NewShell(command),
 			)
 		},
 	})
@@ -913,13 +944,15 @@ func (gui *Gui) handleCreateStashMenu() error {
 		{
 			displayString: gui.Tr.LcStashAllChanges,
 			onPress: func() error {
-				return gui.handleStashSave(gui.GitCommand.WithSpan(gui.Tr.Spans.StashAllChanges).StashSave)
+				gui.logAction(gui.Tr.Actions.StashAllChanges)
+				return gui.handleStashSave(gui.Git.Stash.Save)
 			},
 		},
 		{
 			displayString: gui.Tr.LcStashStagedChanges,
 			onPress: func() error {
-				return gui.handleStashSave(gui.GitCommand.WithSpan(gui.Tr.Spans.StashStagedChanges).StashSaveStagedChanges)
+				gui.logAction(gui.Tr.Actions.StashStagedChanges)
+				return gui.handleStashSave(gui.Git.Stash.SaveStagedChanges)
 			},
 		},
 	}
@@ -928,7 +961,7 @@ func (gui *Gui) handleCreateStashMenu() error {
 }
 
 func (gui *Gui) handleStashChanges() error {
-	return gui.handleStashSave(gui.GitCommand.StashSave)
+	return gui.handleStashSave(gui.Git.Stash.Save)
 }
 
 func (gui *Gui) handleCreateResetToUpstreamMenu() error {
@@ -982,8 +1015,9 @@ func (gui *Gui) handleOpenMergeTool() error {
 		title:  gui.Tr.MergeToolTitle,
 		prompt: gui.Tr.MergeToolPrompt,
 		handleConfirm: func() error {
+			gui.logAction(gui.Tr.Actions.OpenMergeTool)
 			return gui.runSubprocessWithSuspenseAndRefresh(
-				gui.OSCommand.ExecutableFromString(gui.GitCommand.OpenMergeToolCmd()),
+				gui.Git.WorkingTree.OpenMergeToolCmdObj(),
 			)
 		},
 	})
