@@ -7,14 +7,15 @@ import (
 	"os"
 	"sync"
 
-	"os/exec"
 	"strings"
 	"time"
 
 	"github.com/jesseduffield/gocui"
 	"github.com/jesseduffield/lazygit/pkg/commands"
+	"github.com/jesseduffield/lazygit/pkg/commands/git_config"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
 	"github.com/jesseduffield/lazygit/pkg/commands/oscommands"
+	"github.com/jesseduffield/lazygit/pkg/common"
 	"github.com/jesseduffield/lazygit/pkg/config"
 	"github.com/jesseduffield/lazygit/pkg/gui/filetree"
 	"github.com/jesseduffield/lazygit/pkg/gui/lbl"
@@ -26,12 +27,10 @@ import (
 	"github.com/jesseduffield/lazygit/pkg/gui/presentation/graph"
 	"github.com/jesseduffield/lazygit/pkg/gui/style"
 	"github.com/jesseduffield/lazygit/pkg/gui/types"
-	"github.com/jesseduffield/lazygit/pkg/i18n"
 	"github.com/jesseduffield/lazygit/pkg/tasks"
 	"github.com/jesseduffield/lazygit/pkg/theme"
 	"github.com/jesseduffield/lazygit/pkg/updates"
 	"github.com/jesseduffield/lazygit/pkg/utils"
-	"github.com/sirupsen/logrus"
 	"gopkg.in/ozeidan/fuzzy-patricia.v3/patricia"
 )
 
@@ -68,10 +67,10 @@ type Repo string
 
 // Gui wraps the gocui Gui object which handles rendering and events
 type Gui struct {
-	g          *gocui.Gui
-	Log        *logrus.Entry
-	GitCommand *commands.GitCommand
-	OSCommand  *oscommands.OSCommand
+	*common.Common
+	g         *gocui.Gui
+	Git       *commands.GitCommand
+	OSCommand *oscommands.OSCommand
 
 	// this is the state of the GUI for the current repo
 	State *guiState
@@ -80,7 +79,6 @@ type Gui struct {
 	// gui state when returning from a subrepo
 	RepoStateMap         map[Repo]*guiState
 	Config               config.AppConfigurer
-	Tr                   *i18n.TranslationSet
 	Updater              *updates.Updater
 	statusManager        *statusManager
 	credentials          credentials
@@ -114,13 +112,16 @@ type Gui struct {
 	PauseBackgroundThreads bool
 
 	// Log of the commands that get run, to be displayed to the user.
-	CmdLog       []string
-	OnRunCommand func(entry oscommands.CmdLogEntry)
+	CmdLog []string
 
 	// the extras window contains things like the command log
 	ShowExtrasWindow bool
 
 	suggestionsAsyncHandler *tasks.AsyncHandler
+
+	PopupHandler PopupHandler
+
+	IsNewRepo bool
 }
 
 type listPanelState struct {
@@ -346,6 +347,9 @@ type guiState struct {
 
 	// for displaying suggestions while typing in a file name
 	FilesTrie *patricia.Trie
+
+	// this is the message of the last failed commit attempt
+	failedCommitMessage string
 }
 
 // reuseState determines if we pull the repo state from our repo state map or
@@ -372,7 +376,7 @@ func (gui *Gui) resetState(filterPath string, reuseState bool) {
 		}
 	}
 
-	showTree := gui.Config.GetUserConfig().Gui.ShowFileTree
+	showTree := gui.UserConfig.Gui.ShowFileTree
 
 	contexts := gui.contextTree()
 
@@ -430,13 +434,17 @@ func (gui *Gui) resetState(filterPath string, reuseState bool) {
 
 // for now the split view will always be on
 // NewGui builds a new gui handler
-func NewGui(log *logrus.Entry, gitCommand *commands.GitCommand, oSCommand *oscommands.OSCommand, tr *i18n.TranslationSet, config config.AppConfigurer, updater *updates.Updater, filterPath string, showRecentRepos bool) (*Gui, error) {
+func NewGui(
+	cmn *common.Common,
+	config config.AppConfigurer,
+	gitConfig git_config.IGitConfig,
+	updater *updates.Updater,
+	filterPath string,
+	showRecentRepos bool,
+) (*Gui, error) {
 	gui := &Gui{
-		Log:                     log,
-		GitCommand:              gitCommand,
-		OSCommand:               oSCommand,
+		Common:                  cmn,
 		Config:                  config,
-		Tr:                      tr,
 		Updater:                 updater,
 		statusManager:           &statusManager{},
 		viewBufferManagerMap:    map[string]*tasks.ViewBufferManager{},
@@ -444,19 +452,41 @@ func NewGui(log *logrus.Entry, gitCommand *commands.GitCommand, oSCommand *oscom
 		RepoPathStack:           []string{},
 		RepoStateMap:            map[Repo]*guiState{},
 		CmdLog:                  []string{},
-		ShowExtrasWindow:        config.ShowCommandLogOnStartup(),
 		suggestionsAsyncHandler: tasks.NewAsyncHandler(),
+
+		// originally we could only hide the command log permanently via the config
+		// but now we do it via state. So we need to still support the config for the
+		// sake of backwards compatibility. We're making use of short circuiting here
+		ShowExtrasWindow: cmn.UserConfig.Gui.ShowCommandLog && !config.GetAppState().HideCommandLog,
+	}
+
+	guiIO := oscommands.NewGuiIO(
+		cmn.Log,
+		gui.logCommand,
+		gui.getCmdWriter,
+		gui.promptUserForCredential,
+	)
+
+	osCommand := oscommands.NewOSCommand(cmn, oscommands.GetPlatform(), guiIO)
+
+	gui.OSCommand = osCommand
+	var err error
+	gui.Git, err = commands.NewGitCommand(
+		cmn,
+		osCommand,
+		gitConfig,
+	)
+	if err != nil {
+		return nil, err
 	}
 
 	gui.resetState(filterPath, false)
 
 	gui.watchFilesForChanges()
 
-	onRunCommand := gui.GetOnRunCommand()
-	oSCommand.SetOnRunCommand(onRunCommand)
-	gui.OnRunCommand = onRunCommand
+	gui.PopupHandler = &RealPopupHandler{gui: gui}
 
-	authors.SetCustomAuthors(gui.Config.GetUserConfig().Gui.AuthorColors)
+	authors.SetCustomAuthors(gui.UserConfig.Gui.AuthorColors)
 
 	return gui, nil
 }
@@ -506,7 +536,7 @@ func (gui *Gui) Run() error {
 	if err := gui.Config.ReloadUserConfig(); err != nil {
 		return nil
 	}
-	userConfig := gui.Config.GetUserConfig()
+	userConfig := gui.UserConfig
 	g.SearchEscapeKey = gui.getKey(userConfig.Keybinding.Universal.Return)
 	g.NextSearchMatchKey = gui.getKey(userConfig.Keybinding.Universal.NextMatch)
 	g.PrevSearchMatchKey = gui.getKey(userConfig.Keybinding.Universal.PrevMatch)
@@ -522,7 +552,7 @@ func (gui *Gui) Run() error {
 	}
 
 	gui.waitForIntro.Add(1)
-	if gui.Config.GetUserConfig().Git.AutoFetch {
+	if gui.UserConfig.Git.AutoFetch {
 		go utils.Safe(gui.startBackgroundFetch)
 	}
 
@@ -575,7 +605,7 @@ func (gui *Gui) RunAndHandleError() error {
 }
 
 // returns whether command exited without error or not
-func (gui *Gui) runSubprocessWithSuspenseAndRefresh(subprocess *exec.Cmd) error {
+func (gui *Gui) runSubprocessWithSuspenseAndRefresh(subprocess oscommands.ICmdObj) error {
 	_, err := gui.runSubprocessWithSuspense(subprocess)
 	if err != nil {
 		return err
@@ -589,7 +619,7 @@ func (gui *Gui) runSubprocessWithSuspenseAndRefresh(subprocess *exec.Cmd) error
 }
 
 // returns whether command exited without error or not
-func (gui *Gui) runSubprocessWithSuspense(subprocess *exec.Cmd) (bool, error) {
+func (gui *Gui) runSubprocessWithSuspense(subprocess oscommands.ICmdObj) (bool, error) {
 	gui.Mutexes.SubprocessMutex.Lock()
 	defer gui.Mutexes.SubprocessMutex.Unlock()
 
@@ -618,27 +648,26 @@ func (gui *Gui) runSubprocessWithSuspense(subprocess *exec.Cmd) (bool, error) {
 	return cmdErr == nil, gui.surfaceError(cmdErr)
 }
 
-func (gui *Gui) runSubprocess(subprocess *exec.Cmd) error {
+func (gui *Gui) runSubprocess(cmdObj oscommands.ICmdObj) error { //nolint:unparam
+	gui.logCommand(cmdObj.ToString(), true)
+
+	subprocess := cmdObj.GetCmd()
 	subprocess.Stdout = os.Stdout
 	subprocess.Stderr = os.Stdout
 	subprocess.Stdin = os.Stdin
 
 	fmt.Fprintf(os.Stdout, "\n%s\n\n", style.FgBlue.Sprint("+ "+strings.Join(subprocess.Args, " ")))
 
-	if err := subprocess.Run(); err != nil {
-		// not handling the error explicitly because usually we're going to see it
-		// in the output anyway
-		gui.Log.Error(err)
-	}
+	err := subprocess.Run()
 
 	subprocess.Stdout = ioutil.Discard
 	subprocess.Stderr = ioutil.Discard
 	subprocess.Stdin = nil
 
-	fmt.Fprintf(os.Stdout, "\n%s", style.FgGreen.Sprint(gui.Tr.PressEnterToReturn))
+	fmt.Fprintf(os.Stdout, "\n%s\n", style.FgGreen.Sprint(gui.Tr.PressEnterToReturn))
 	fmt.Scanln() // wait for enter press
 
-	return nil
+	return err
 }
 
 func (gui *Gui) loadNewRepo() error {
@@ -707,12 +736,12 @@ func (gui *Gui) goEvery(interval time.Duration, stop chan struct{}, function fun
 
 func (gui *Gui) startBackgroundFetch() {
 	gui.waitForIntro.Wait()
-	isNew := gui.Config.GetIsNewRepo()
-	userConfig := gui.Config.GetUserConfig()
+	isNew := gui.IsNewRepo
+	userConfig := gui.UserConfig
 	if !isNew {
 		time.After(time.Duration(userConfig.Refresher.FetchInterval) * time.Second)
 	}
-	err := gui.fetch(false, "")
+	err := gui.backgroundFetch()
 	if err != nil && strings.Contains(err.Error(), "exit status 128") && isNew {
 		_ = gui.ask(askOpts{
 			title:  gui.Tr.NoAutomaticGitFetchTitle,
@@ -720,7 +749,7 @@ func (gui *Gui) startBackgroundFetch() {
 		})
 	} else {
 		gui.goEvery(time.Second*time.Duration(userConfig.Refresher.FetchInterval), gui.stopChan, func() error {
-			err := gui.fetch(false, "")
+			err := gui.backgroundFetch()
 			gui.render()
 			return err
 		})
@@ -729,7 +758,7 @@ func (gui *Gui) startBackgroundFetch() {
 
 // setColorScheme sets the color scheme for the app based on the user config
 func (gui *Gui) setColorScheme() error {
-	userConfig := gui.Config.GetUserConfig()
+	userConfig := gui.UserConfig
 	theme.UpdateTheme(userConfig.Gui.Theme)
 
 	gui.g.FgColor = theme.InactiveBorderColor
