@@ -5,7 +5,6 @@ import (
 	"fmt"
 	"strings"
 
-	"github.com/jesseduffield/gocui"
 	"github.com/jesseduffield/lazygit/pkg/theme"
 	"github.com/jesseduffield/lazygit/pkg/utils"
 )
@@ -27,16 +26,10 @@ func (i *menuItem) ID() string {
 	return strings.Join(i.displayStrings, "-")
 }
 
-// list panel functions
-
-func (gui *Gui) handleMenuSelect() error {
-	return nil
-}
-
 // specific functions
 
 func (gui *Gui) getMenuOptions() map[string]string {
-	keybindingConfig := gui.Config.GetUserConfig().Keybinding
+	keybindingConfig := gui.UserConfig.Keybinding
 
 	return map[string]string{
 		gui.getKeyDisplay(keybindingConfig.Universal.Return): gui.Tr.LcClose,
@@ -95,10 +88,7 @@ func (gui *Gui) createMenu(title string, items []*menuItem, createMenuOptions cr
 	menuView.SetContent(list)
 	gui.State.Panels.Menu.SelectedLineIdx = 0
 
-	gui.g.Update(func(g *gocui.Gui) error {
-		return gui.pushContext(gui.State.Contexts.Menu)
-	})
-	return nil
+	return gui.pushContext(gui.State.Contexts.Menu)
 }
 
 func (gui *Gui) onMenuPress() error {
