@@ -43,7 +43,7 @@ func (gui *Gui) handleSearch() error {
 }
 
 func (gui *Gui) onSelectItemWrapper(innerFunc func(int) error) func(int, int, int) error {
-	keybindingConfig := gui.Config.GetUserConfig().Keybinding
+	keybindingConfig := gui.UserConfig.Keybinding
 
 	return func(y int, index int, total int) error {
 		if total == 0 {
