@@ -44,11 +44,11 @@ func polishTranslationSet() TranslationSet {
 		SureForceCheckout:                   "Jesteś pewny, że chcesz wymusić przełączenie? Stracisz wszystkie lokalne zmiany",
 		ForceCheckoutBranch:                 "Wymuś przełączenie gałęzi",
 		BranchName:                          "Nazwa gałęzi",
-		NewBranchNameBranchOff:              "Nazwa nowej gałęzi (gałąź na bazie {{.branchName}})",
+		NewBranchNameBranchOff:              "Nazwa nowej gałęzi (gałąź na bazie '{{.branchName}}')",
 		CantDeleteCheckOutBranch:            "Nie możesz usunąć obecnie przełączonej gałęzi!",
 		DeleteBranch:                        "Usuń gałąź",
-		DeleteBranchMessage:                 "Jesteś pewien, że chcesz usunąć gałąź {{.selectedBranchName}} ?",
-		ForceDeleteBranchMessage:            "Na pewno wymusić usunięcie gałęzi {{.selectedBranchName}}?",
+		DeleteBranchMessage:                 "Jesteś pewien, że chcesz usunąć gałąź '{{.selectedBranchName}}' ?",
+		ForceDeleteBranchMessage:            "Na pewno wymusić usunięcie gałęzi '{{.selectedBranchName}}'?",
 		LcRebaseBranch:                      "zmiana bazy gałęzi",
 		CantRebaseOntoSelf:                  "Nie możesz zmienić bazy gałęzi na nią samą",
 		CantMergeBranchIntoItself:           "Nie możesz scalić gałęzi do samej siebie",
@@ -69,8 +69,7 @@ func polishTranslationSet() TranslationSet {
 		YouNoCommitsToSquash:                "Nie masz commitów do spłaszczenia",
 		Fixup:                               "Napraw",
 		SureFixupThisCommit:                 "Jesteś pewny, ze chcesz naprawić ten commit? Commit poniżej zostanie spłaszczony w górę wraz z tym",
-		OnlyRenameTopCommit:                 "Można zmienić nazwę tylko ostatniemu commitowi",
-		LcRenameCommit:                      "zmień nazwę commita",
+		LcRewordCommit:                      "zmień nazwę commita",
 		LcRenameCommitEditor:                "zmień nazwę commita w edytorze",
 		Error:                               "Błąd",
 		LcSelectHunk:                        "wybierz kawałek",
@@ -84,7 +83,6 @@ func polishTranslationSet() TranslationSet {
 		NoStashEntries:                      "Brak pozycji w schowku",
 		StashDrop:                           "Porzuć schowek",
 		SureDropStashEntry:                  "Jesteś pewny, że chcesz porzucić tę pozycję w schowku?",
-		NoStashTo:                           "Brak schowka dla {{.method}}",
 		NoTrackedStagedFilesStash:           "Nie masz śledzonych/zatwierdzonych plików do przechowania",
 		StashChanges:                        "Przechowaj zmiany",
 		MergeAborted:                        "Scalanie anulowane",
@@ -119,8 +117,8 @@ func polishTranslationSet() TranslationSet {
 		ReturnToFilesPanel:                  "wróć do panelu plików",
 		RebasingTitle:                       "Zmiana bazy",
 		MergingTitle:                        "Scalanie",
-		ConfirmRebase:                       "Czy napewno chcesz zmienić bazę {{.checkedOutBranch}} na {{.selectedBranch}}?",
-		ConfirmMerge:                        "Czy na pewno chcesz scalić {{.selectedBranch}} do {{.checkedOutBranch}}?",
+		ConfirmRebase:                       "Czy napewno chcesz zmienić bazę '{{.checkedOutBranch}}' na '{{.selectedBranch}}'?",
+		ConfirmMerge:                        "Czy na pewno chcesz scalić '{{.selectedBranch}}' do '{{.checkedOutBranch}}'?",
 		FwdNoUpstream:                       "Nie można przewinąć gałęzi bez gałęzi nadrzędnej",
 		FwdCommitsToPush:                    "Nie można przewinąć gałęzi z commitami do wysłania",
 		ErrorOccurred:                       "Wystąpił błąd! Zgłoś problem na",
